/*  This file is a part of Puzzles, a MIDLET for playing puzzles.
    Copyright (C) 2004, Kostas Giannakakis

    Puzzles is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Puzzles is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Puzzles; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/*
 * @(#)Board.java	1.0 04/02/18
 *
 */

package games.Q8Puzzle;

import games.*;
import java.util.Date;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;
import java.io.IOException;

public class Board extends Canvas implements CommandListener
{

	MIDlet midlet;
	Display dpy;

	/** Table for storing queens positions. */
	int [] posArray;

	/** Position of selected square 0..63 */
	int selPos;

	/** The image for queen at white square. */
	Image queenWhiteSquareImage;

    /** The image for queen at black square. */
    Image queenBlackSquareImage;

	// commands
	static final int CMD_ABOUT	= 0;
	static final int CMD_EXIT	= 1;
	static final int CMD_RESET	= 2;
	static final int CMD_BEST_TIME	= 3;
	static final int CMD_RESTART	= 4;
	static final int CMD_ZLAST	= 5;	// must be ze last, of course
	Command cmd[];

	// state variables
	static final int PLAYING = 0;
	static final int WON = 1;
	static final int ENDED = 2;
	int gameState;

	// Drawing constants
	static final int TILE_SIZE = 11;
	static final int BOARD_SIZE = 88;
	static final int X_OFFSET = 20;
	static final int Y_OFFSET = 4;

	// Time variables
	Date startTime;
	Date endTime;

	/*void D(String s)
	{
		System.out.println(s);
	}*/


	class BoardCommand extends Command
	{
		int tag;

		BoardCommand(String label, int type, int pri, int tag_)
		{
			super(label, type, pri);
			tag = tag_;
		}
	}

	/**
	 * Initialises board.
	 */
    public void initBoard()
	{
    	resetBoard(true);

    	try
    	{
    		queenWhiteSquareImage =
    			Image.createImage("/games/q8puzzle/images/queenw.png");
			queenBlackSquareImage =
	    		Image.createImage("/games/q8puzzle/images/queenb.png");
		}
		catch(IOException e){}
    }

	/**
	 * Resets board's variables.
	 */
    public void resetBoard(boolean resetTime)
	{
    	posArray = new int[8];
    	for(int i =0; i<8; i++)
    		posArray[i] = -1;

    	selPos = 0;

		if (resetTime)
			startTime = new Date();
    }

	/**
	 * Sets game's state. Updates commands list.
	 */
	void setState(int ns)
	{
		gameState = ns;
		removeCommand(cmd[CMD_RESET]);
		removeCommand(cmd[CMD_RESTART]);
		removeCommand(cmd[CMD_BEST_TIME]);
		removeCommand(cmd[CMD_ABOUT]);
		removeCommand(cmd[CMD_EXIT]);
		switch (gameState) {
		case PLAYING:

			addCommand(cmd[CMD_RESET]);
			addCommand(cmd[CMD_BEST_TIME]);
			addCommand(cmd[CMD_ABOUT]);
			addCommand(cmd[CMD_EXIT]);
			break;
		case WON:
		case ENDED:
			addCommand(cmd[CMD_RESTART]);
			addCommand(cmd[CMD_BEST_TIME]);
			addCommand(cmd[CMD_ABOUT]);
			addCommand(cmd[CMD_EXIT]);
			break;
		}
	}

	/**
	 * Stores midlet, display variables.
	 * Set up commands, initialises board.
	 */
	public Board(MIDlet midlet_)
	{
		int i;

		// "global" variables
		midlet = midlet_;
		dpy = Display.getDisplay(midlet);

		// set up commands
		cmd = new Command[CMD_ZLAST];

		cmd[CMD_ABOUT] =
			new BoardCommand("About", Command.HELP, 3, CMD_ABOUT);

		cmd[CMD_EXIT] =
			new BoardCommand("Exit", Command.EXIT, 4, CMD_EXIT);

		cmd[CMD_BEST_TIME] =
			new BoardCommand("Best Time", Command.SCREEN, 2, CMD_BEST_TIME);

		cmd[CMD_RESTART] =
			new BoardCommand("Restart", Command.SCREEN, 1, CMD_RESTART);

		cmd[CMD_RESET] =
			new BoardCommand("Reset", Command.SCREEN, 1, CMD_RESET);


		// set up the listener
		setCommandListener(this);

		// set up initial state
		setState(PLAYING);

		// Init Board
		initBoard();
	}

	/**
	 * Command actions handling.
	 */
	public void commandAction(Command c, Displayable d) {
		switch (((BoardCommand) c).tag) {
			case CMD_ABOUT:
				About.showAbout(Display.getDisplay(midlet));
				break;
			case CMD_EXIT:
				midlet.notifyDestroyed();
				break;
			case CMD_RESET:
				setState(PLAYING);
				resetBoard(false);
				repaint();
				break;
			case CMD_RESTART:
				setState(PLAYING);
				resetBoard(true);
				repaint();
				break;
			case CMD_BEST_TIME:
				long bestTime = 0;
				Score score = new Score(1);
				if (score.open("Q8Scores"))
				{
					bestTime = score.getTime(0);
				}
				score.close();
				long sec = (bestTime / 1000) % 60,
					 min = (bestTime / 1000) /60;
				String str = new String();
				if (bestTime == 0)
					str = "Not solved";
				else if (sec >= 10)
					str +=  min + ":" + sec;
				else
					str +=  min + ":0" + sec;

				Alert alert = new Alert("Eight Queens");
				alert.setTimeout(Alert.FOREVER);
				alert.setString(str);
				dpy.setCurrent(alert);
				break;
		}
	}

	/**
	 * Paint routine
	 */
    public void paint(Graphics g)
	{
		g.setColor(0xFFFFFF);
		g.fillRect(0, 0, getWidth(), getHeight());

		g.setColor(0);

		int r = selPos / 8,
		    c = selPos % 8;

		for(int i=7; i>=0; i--)
		{
			for(int j=0;j<8;j++)
			{
				if((i+j)%2==0)
				{
					// White square
					if(posArray[j]==i)
						g.drawImage(queenWhiteSquareImage,
									X_OFFSET + j*TILE_SIZE,
									Y_OFFSET + i*TILE_SIZE,
									0);
				}
				else
				{
					// Black square
					g.fillRect(X_OFFSET + j*TILE_SIZE,
							   Y_OFFSET + i*TILE_SIZE,
							   TILE_SIZE,
							   TILE_SIZE);

					if(posArray[j]==i)
						g.drawImage(queenBlackSquareImage,
							    	X_OFFSET + j*TILE_SIZE,
									Y_OFFSET + i*TILE_SIZE,
									0);
				}
			}
			// Selected square
			selectSquare(g, r, c, ((r+c)%2==0));

			g.drawRect(X_OFFSET, Y_OFFSET, BOARD_SIZE - 1, BOARD_SIZE - 1);
		}

		if (gameState == WON)
		{
			Alert alert = new Alert("Well done!");
			alert.setTimeout(Alert.FOREVER);

			long time = endTime.getTime() - startTime.getTime();
			long bestTime = 0;

			// Read best time, update if necessary
			Score score = new Score(1);
			if (score.open("Q8Scores"))
			{
				bestTime = score.getTime(0);
				if (bestTime > time || bestTime == 0)
					score.setTime(0, (int) time);
			}
			score.close();

			long sec = (time / 1000) % 60,
				 min = (time / 1000) /60;

			String str = new String();
			if (bestTime > time || bestTime == 0)
				str = "New best time!\n";
			else
				str = "Solved\nTime: ";
			if (sec >= 10)
				str +=  min + ":" + sec;
			else
				str +=  min + ":0" + sec;
			alert.setString(str);

			Display.getDisplay(midlet).setCurrent(alert);
			gameState = ENDED;
		}
    }

    private void selectSquare(Graphics g,
                              int r,
                              int c,
                              boolean isWhite)
    {
		int oldColor = g.getColor();

		g.setColor(0xFF0000);
		g.drawRect(X_OFFSET - 1 + c*TILE_SIZE,
				   Y_OFFSET - 1 + r*TILE_SIZE,
				   TILE_SIZE + 1,
				   TILE_SIZE + 1);
		g.setColor(oldColor);
	}


	/**
	 * Key Pressed handling
	 */
	public void keyPressed(int code)
	{
		if (gameState != PLAYING)
			return;

		int game = getGameAction(code);

		switch (game)
		{
			case Canvas.UP:
				if (selPos > 7)
					selPos-=8;
				else
					selPos+=56;
				break;
			case Canvas.DOWN:
				if (selPos < 56)
					selPos+=8;
				else
					selPos-=56;
				break;
			case Canvas.LEFT:
				if (selPos % 8 == 0)
					selPos+=7;
				else
					selPos--;
				break;
			case Canvas.RIGHT:
				selPos++;
				if (selPos % 8 == 0)
					selPos-=8;
				break;
			case Canvas.FIRE:
				int r = selPos / 8,
				    c = selPos % 8;

				makeMove(r, c);
				break;
			default:
			return;
		}

		repaint();

		if (isSolved())
		{
			endTime = new Date();
			setState(WON);
		}
	}

	/**
	 * Puzzle's routine. Square at row r, column c is selected
	 * If a queen is already placed there remove it. If the
	 * square is empty and it is allowed to place a queen there,
	 * do it!
	 */
    public void makeMove(int r, int c)
	{
		if (c < 8 && r < 8)
		{
			if (posArray[c] == -1)
			{
				posArray[c] = r;
				if (queensPlaced()==0)
				{
					posArray[c] = -1;
					//play(getCodeBase(), "audio/beep.au");
				}
			}
			else if (posArray[c] == r)
			{
				posArray[c] = -1;
			}
			else
			{
				//play(getCodeBase(), "audio/beep.au");
			}
		}
    }

	/**
	 * Puzzle's routine. Returns the number of queens placed.
	 */
    private int queensPlaced()
	{
    	int i, j, queens = 0;

    	for(i=0;i<8;i++)
    	{
    		if (posArray[i]!=-1)
    		{
    			queens++;
    			for(j=i+1;j<8;j++)
	    		{
	    			if (posArray[j]==-1)
	    				continue;
	    			if (posArray[i] == posArray[j])
	    				return(0);
	    			if (i + posArray[i] == j + posArray[j])
	    				return(0);
	    			if (i - posArray[i] == j - posArray[j])
	    				return(0);
	    		}
    		}
    	}
    	return(queens);
    }

	/**
	 * Puzzle's routine. Returns true if the puzzle is solved.
	 */
	boolean isSolved()
	{
		return(queensPlaced() == 8);
	}
}
