/*  This file is a part of Puzzles, a MIDLET for playing puzzles.
    Copyright (C) 2004, Kostas Giannakakis

    Puzzles is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Puzzles is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Puzzles; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/*
 * @(#)Score.java	1.0 05/04/14
 *
 */

package games;

import javax.microedition.rms.*;

public class Score {

	private static int scoreCount;
	private static final int STORE_LEN = 5;
	private int[] boardTimes;
	private int levelId;
    private RecordStore store;		// Record store, null if not open


    /*
     * Construct a new Score handler.
     */
    public Score(int count)
    {
		scoreCount = count;
		store = null;
		boardTimes = new int[scoreCount];
		levelId = 0;
    }

    /**
     * Open the record store
     */
    public boolean open(String recordStoreName)
    {
		try {
			store = RecordStore.openRecordStore(recordStoreName, true);
		} catch (RecordStoreException ex) {
		}

		if (store == null)
			return false;

		for(int i = 0; i < scoreCount; i++)
		{
			boardTimes[i] = 0;
		}

		try
		{
			RecordEnumeration enum = store.enumerateRecords(null, null, false);
			while (enum.hasNextElement())
			{
				int index = enum.nextRecordId();
				if (store.getRecordSize(index) == STORE_LEN)
				{
					byte buf[] = new byte[STORE_LEN];
					int length = store.getRecord(index, buf, 0);
					if (length == STORE_LEN)
					{
						int level = buf[0];
						if (level >= 0 && level < scoreCount)
							boardTimes[level] = getInt(buf, 1);
					}
				}
			}
		} catch (RecordStoreException ex) {
			ex.printStackTrace();
			return false;
		}
		return true;
    }

	public boolean getStatus(int index)
	{
		if (index >= scoreCount || index < 0)
			return false;
		return boardTimes[index] > 0;
	}

	public boolean setTime(int level, int time)
	{
		// getTime(level) must have been performed before

		if (level >= scoreCount || level < 0)
			return false;

		byte buf[] = new byte[STORE_LEN];

		buf[0] = (byte) level;
		putInt(buf, 1, time);

		try
		{
			if (levelId == 0)
			{
				levelId = store.addRecord(buf, 0, STORE_LEN);
			}
			else
			{
				store.setRecord(levelId, buf, 0, STORE_LEN);
			}
			boardTimes[level] = time;
		}
		catch (RecordStoreException ex)
		{
		    System.out.println("RecordStoreException");
		    ex.printStackTrace();
		    return false;
		}
		return true;
	}

	public int getTime(int level)
	{
		if (level >= scoreCount || level < 0)
			return 0;

		levelId = 0;
		try
		{
			// Locate the matching record
			RecordEnumeration enum = store.enumerateRecords(null, null, false);
			while (enum.hasNextElement())
			{
				int index = enum.nextRecordId();
				byte buf[] = new byte[STORE_LEN];
				if (store.getRecordSize(index) == STORE_LEN)
				{
			  		int length = store.getRecord(index, buf, 0);
			  		if (length == STORE_LEN &&
				  		buf[0] == level)
				  	{
						levelId = index;
						return getInt(buf, 1);
			  		}
				}
			}
		}
		catch (RecordStoreException ex)
		{
			ex.printStackTrace();

		}
		return 0;
	}

    /**
     * Get an integer from an array.
     */
    private int getInt(byte[] buf, int offset)
    {
		return 	(buf[offset+0] & 0xff) << 24 |
				(buf[offset+1] & 0xff) << 16 |
				(buf[offset+2] & 0xff) << 8 |
				(buf[offset+3] & 0xff);
    }

    /**
     * Put an integer to an array
     */
    private void putInt(byte[] buf, int offset, int value)
    {
		buf[offset+0] = (byte)((value >> 24) & 0xff);
		buf[offset+1] = (byte)((value >> 16) & 0xff);
		buf[offset+2] = (byte)((value >>  8) & 0xff);
		buf[offset+3] = (byte)((value >>  0) & 0xff);
    }


    public void close()
    {
		try {
			if (store != null) {
			store.closeRecordStore();
			}
		} catch (RecordStoreException ex) {
			ex.printStackTrace();
		}
    }
}
